local addonName, vars = ...
local L = vars.L
AscendanceTimer = vars
local addon = vars

local settings

local hiddenFrame = CreateFrame("Button", addonName.."HiddenFrame", UIParent)
local revision = tonumber(("$Revision: 2 $"):match("%d+"))
local LDB, LDBo

local SPELLID_ASCENDANCE = 114049
local SPELLID_LAVABURST = 51505
local SPELLID_LAVABURST_OVERLOAD = 77451
local NAME_LB = GetSpellInfo(51505)

local NOMINALCD = 3*60
local SETREDUCE = 1.5   -- tooltip is WRONG: it's actually 1.5 sec per LvB
local playerGUID

addon.expires = 0
addon.lbcnt = {}
addon.lastascend = 0

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage("|cffffff40"..addonName.."|r: "..msg)
end

local function debug(msg)
  if settings and settings.debug then
     chatMsg(msg)
  end
end

local function timefmt(t)
  local min = math.floor(t / 60)
  local sec = math.floor(t % 60)
  return string.format("%d:%02d",min,sec)
end

function addon:Update()
  local now = GetTime()
  local start, duration = GetSpellCooldown(SPELLID_ASCENDANCE)
  if start and duration and start > 0 and duration > 0 then
    local exp = start + duration
    if math.abs(exp - addon.expires) > 3 and exp >= now then
      debug("Adjusting CD: "..string.format("%0.2f",exp-addon.expires).." sec")
      addon.expires = exp
    end
  end
  local remain = addon.expires - now
  if remain <= 0 then
    LDBo.text = READY
    if not addon.ready and addon.lastascend > 0 then
      local cdtime = now - addon.lastascend
      local msg = L["Ascendance cooldown completed after: "]..timefmt(cdtime)..
         "  ("..(addon.lbcnt[SPELLID_LAVABURST] or 0).." "..NAME_LB..", "..
                (addon.lbcnt[SPELLID_LAVABURST_OVERLOAD] or 0).." "..L["Overload"]..")"
      (addon.stats and chatMsg or debug)(msg)
    end
    addon.ready = true
  else
    LDBo.text = timefmt(remain)
    addon.ready = false
  end
end

local function OnUpdate(_, elap)
  addon.elap = (addon.elap or 0) - elap
  if addon.elap < 0 then
     addon.elap = 0.25
     addon:Update()
  end
end

local tier_slots = { INVSLOT_HEAD, INVSLOT_SHOULDER, INVSLOT_CHEST, INVSLOT_LEGS, INVSLOT_HAND }
local tier_pieces = {
  [95951]=1,[95950]=1,[95952]=1,[95953]=1,[95954]=1,
  [95321]=1,[95320]=1,[95322]=1,[95323]=1,[95324]=1,
  [96695]=1,[96694]=1,[96696]=1,[96697]=1,[96698]=1,
}

function addon:check_tier()
    local cnt = 0
    for _, slot in pairs(tier_slots) do
       local itemid = GetInventoryItemID("player", slot)
       if itemid and tier_pieces[itemid] then
         cnt = cnt + 1
       end
    end
    addon.T15set = (cnt >= 4)
    if addon.T15set then
      debug("T15 set bonus enabled")
    else
      debug("T15 set bonus disabled")
    end
end

local function OnEvent(_, event, ...)
  if event == "PLAYER_REGEN_DISABLED" then
    addon:check_tier()
  elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
    local unit, spellname, _, _, spellid = ...
    if unit ~= "player" then return end
    if spellid == SPELLID_ASCENDANCE then
      local now = GetTime()
      debug(spellname)
      addon.expires = now + NOMINALCD
      addon.lastascend = now
      wipe(addon.lbcnt)
      addon:Update()
    end
  elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
    local _, subevent, _, srcGUID = ...
    if subevent == "SPELL_DAMAGE" and srcGUID == playerGUID then
      local spellid = select(12, ...)
      if spellid == SPELLID_LAVABURST or spellid == SPELLID_LAVABURST_OVERLOAD then
        --debug(spellid)
	addon.lbcnt[spellid] = (addon.lbcnt[spellid] or 0) + 1
        if addon.T15set then
          addon.expires = addon.expires - SETREDUCE
          addon:Update()
        end
      end
    end
  end
end

function addon:OnEnable()
  debug("OnEnable")
  AscendanceTimerDB = AscendanceTimerDB or {}
  settings = AscendanceTimerDB

  hiddenFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
  hiddenFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  hiddenFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
  hiddenFrame:SetScript("OnUpdate",OnUpdate)
  hiddenFrame:SetScript("OnEvent",OnEvent)

  playerGUID = UnitGUID("player")
 
  if LDB then
    return
  end
  LDB = LibStub:GetLibrary("LibDataBroker-1.1")
  LDBo = LDB:NewDataObject(addonName, {
        type = "data source",
        label = addonName,
	text = READY,
        icon = "Interface\\Icons\\spell_fire_elementaldevastation",
        OnClick = function(self, button)
		local set
                if button == "RightButton" then
			set = "debug"
                else
			set = "stats"
                end
		settings[set] = not settings[set]
		chatMsg(set.." "..(settings[set] and L["enabled"] or L["disabled"]))
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine then
                        tooltip:SetText(addonName)
                        tooltip:AddLine("|cffff8040"..L["Left Click"].."|r "..L["to toggle cooldown stats"])
                        tooltip:AddLine("|cffff8040"..L["Right Click"].."|r "..L["to toggle debugging"])
                        tooltip:Show()
                end
        end,
     })

  addon:check_tier()
  addon:Update()
end

addon:OnEnable()

